/* GLibDocumentController.m created by stark on Fri 13-Nov-1998 */

/* 
   GLibDocumentController.m

   GLibDocumentController for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibDocumentController.h"
#import "GLibDocument.h"
#import "GLibTarget.h"
#import "GLibNotification.h"
#import "GLibQueryResultGroup.h"
#import "GLibQuery.h"
#import "GLibQueryResult.h"
#import "GLibQueryExecutor.h"

@interface GLibDocumentController(GLibPrivate)
	//	Create the window with its UI widgets
- (void)_glibCreateWindow;
	//	Populate the so-called shelf
- (void)_glibUpdateShelf;
	//	Update item in shelf (ugly)
- (void)_glibUpdateShelfAt:(int)aPosition;
	//	Configure the cell to represent the target (which may be nil)
- (void)_glibConfigureCell:(id)aCell forTarget:(GLibTarget *)aTarget;
	//	Start observing a document
- (void)_glibStartObservingDocument:(GLibDocument *)aDocument;
	//	Stop observing a document
- (void)_glibStopObservingDocument:(GLibDocument *)aDocument;
	//	Start observing an array of result group
- (void)_glibStartObservingResultGroupArray:(NSArray *)anArray;
	//	Stop observing an array of result group
- (void)_glibStopObservingResultGroupArray:(NSArray *)anArray;
	//	Copy the user interface into the query [too simple]. Returns NO if query is not valid
- (BOOL)_glibInterfaceToQuery;
	//	Re-create the active targets array from the interface
- (void)_glibInterfaceToActiveTargets;
	//	Rebuild all the result matrix
- (void)_glibRebuildResultMatrix;
	//	Add result from group to matrix
- (void)_glibAppendResultGroup:(GLibQueryResultGroup *)aGroup;
	//	Create and start all the executors for a query
- (void)_glibStartExecutorsForQuery:(GLibQuery *)aQuery resultGroups:(NSArray *)aResultGroupArray;
	//	Check for termination
- (void)_glibCheckForTermination;
	//	Abort all executors
- (void)_glibAbortExecutors;
@end

//	----------------------------------------------------------------------------

@implementation GLibDocumentController

- init
{	[super init];
	itsQuery = [[GLibSimpleQuery alloc] init];		//	#### Too simple
	itsActiveTargets = [[NSMutableArray alloc] init];
	itsExecutors = nil;
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[self _glibStopObservingDocument:itsDocument];
	[self _glibStopObservingResultGroupArray:itsResultGroups];
	[self _glibAbortExecutors];		//	#### This is not enough. Executors may have notification pending. We'd better assert
	[itsQuery release];
	[itsActiveTargets release];
	[itsResultGroups release];
	[itsDocument release];
	assert( !itsExecutors );
	[super dealloc];
}

//	----------------------------------------------------------------------------

- (void)glibUpdateUserInterface
{		//	Populate the shelf
	[self _glibUpdateShelf];
}

//	----------------------------------------------------------------------------

- (void)glibShowUserInterface
{		//	Create the window if necessary
	if (!itsWindow)
		[self _glibCreateWindow];

		//	Update the interface content
	[self glibUpdateUserInterface];

		//	Make window front
	[itsWindow makeKeyAndOrderFront:self];
}

//	----------------------------------------------------------------------------

- (void)glibSetDocument:(GLibDocument *)aDocument
{	[self _glibStopObservingDocument:itsDocument];
	[itsDocument autorelease];
	itsDocument = aDocument;
	[itsDocument retain];
	[self _glibStartObservingDocument:itsDocument];


/*
	[self glibShowUserInterface];
	[itsDocumentMatrix selectCellAtRow:0 column:0];
	[itsQueryField setStringValue:@"xxx"];
	[self glibStartQuery:nil];
*/
}

//	----------------------------------------------------------------------------

- (IBAction)glibStartQuery:sender
{	NSEnumerator *e;
	GLibTarget *theTarget;

	assert( !itsExecutors );
	itsExecutors = [[NSMutableArray alloc] init];

	//	Validate query
	if (![self _glibInterfaceToQuery])
	{	NSBeep();
		return ;
	}

		//	Change the image/action of the start button
	[itsStartButton setImage:[NSImage imageNamed:@"stop_button"]];
	[itsStartButton setAction:@selector(glibStopQuery:)];

	[self _glibInterfaceToActiveTargets];

		//	Stop listening to old result groups
	[self _glibStopObservingResultGroupArray:itsResultGroups];

		//	Create new result group
	[itsResultGroups release];
	itsResultGroups = [[NSMutableArray alloc] init];

		//	Populate result group
	e = [itsActiveTargets objectEnumerator];
	while (theTarget=[e nextObject])
	{	GLibQueryResultGroup *theResultGroup = [[[GLibQueryResultGroup alloc] init] autorelease];
		[theResultGroup glibSetTarget:theTarget];
		[itsResultGroups addObject:theResultGroup];
	}

		//	Listen to them
	[self _glibStartObservingResultGroupArray:itsResultGroups];

		//	Start the various executors
	[self _glibStartExecutorsForQuery:itsQuery resultGroups:itsResultGroups];
}

//	----------------------------------------------------------------------------

- (IBAction)glibStopQuery:sender
{	[itsStartButton setEnabled:NO];
	[self _glibAbortExecutors];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibDocumentController(GLibPrivate)

	//	Needless to say, we NEED InterfaceModeller

	//	Where we want space, we use this
#define SPACE 8

	//	Document
#define DOC_CELL_W			120
#define DOC_CELL_H			80
#define DOC_CELL_CNT_X		5
#define DOC_CELL_CNT_Y		3
#define DOC_MATRIX_W		(DOC_CELL_CNT_X*DOC_CELL_W)
#define DOC_MATRIX_H		(DOC_CELL_CNT_Y*DOC_CELL_H)
#define DOC_MATRIX_X		SPACE
#define DOC_MATRIX_Y		((QUERY_VIEW_Y+QUERY_VIEW_H)+SPACE)

	//	Start button
#define START_BTN_W			48
#define START_BTN_H			48
#define START_BTN_X			((QUERY_VIEW_X+QUERY_VIEW_W)-START_BTN_W)
#define START_BTN_Y			(QUERY_VIEW_Y+(QUERY_VIEW_H-START_BTN_H)/2)

	//	Query
#define QUERY_H	20
#define QUERY_FIELD_W		(QUERY_VIEW_W-START_BTN_W-SPACE)
#define QUERY_FIELD_H		QUERY_H
#define QUERY_FIELD_X		SPACE
#define QUERY_FIELD_Y		(QUERY_VIEW_Y+(QUERY_VIEW_H-QUERY_FIELD_H)/2)

#define QUERY_VIEW_W		RESULT_MATRIX_W
#define QUERY_VIEW_H		(MAX(QUERY_H, START_BTN_H))
#define QUERY_VIEW_X		SPACE
#define QUERY_VIEW_Y		((RESULT_MATRIX_Y+RESULT_MATRIX_H)+SPACE)

	//	Results
#define RESULT_CELL_H		20
#define RESULT_CELL_CNT_Y	16
#define RESULT_MATRIX_W		DOC_MATRIX_W
#define RESULT_MATRIX_H		(RESULT_CELL_H*RESULT_CELL_CNT_Y)
#define RESULT_MATRIX_X		SPACE
#define RESULT_MATRIX_Y		SPACE

	//	WINDOW
#define WINDOW_W			MAX(MAX(RESULT_MATRIX_X+RESULT_MATRIX_W, QUERY_VIEW_X+QUERY_VIEW_W), DOC_MATRIX_X+DOC_MATRIX_W)+SPACE
#define WINDOW_H			DOC_MATRIX_Y+DOC_MATRIX_H+SPACE

- (void)_glibCreateWindow
{
	assert( !itsWindow );

		//	Create the window
	itsWindow = [[NSWindow alloc] initWithContentRect:NSMakeRect( 100, 100, WINDOW_W, WINDOW_H )
						styleMask:NSTitledWindowMask|NSClosableWindowMask|NSMiniaturizableWindowMask|NSResizableWindowMask
						backing:NSBackingStoreBuffered
						defer:YES];

	[itsWindow setTitle:@"GNU Librarian"];

#if 1
		//	Create the document matrix
	itsDocumentMatrix = [[[NSMatrix alloc] initWithFrame:NSMakeRect( DOC_MATRIX_X, DOC_MATRIX_Y, DOC_MATRIX_W, DOC_MATRIX_H )] autorelease];
	[itsDocumentMatrix setPrototype:[[[NSButtonCell alloc] init] autorelease]];

	[[itsDocumentMatrix prototype] setBordered:NO];
	[[itsDocumentMatrix prototype] setImagePosition:NSImageAbove];

	[itsDocumentMatrix setMode:NSListModeMatrix];
/*		As the OPENSTEP implementation of insertColumn: is bugged, we use another way because GNUstep may not be bug-to-bug compatible
	[itsDocumentMatrix insertColumn:DOC_CELL_CNT_X];
	[itsDocumentMatrix insertRow:DOC_CELL_CNT_Y];
*/
	[itsDocumentMatrix renewRows:DOC_CELL_CNT_Y columns:DOC_CELL_CNT_X];

	[itsDocumentMatrix setCellSize:NSMakeSize( DOC_CELL_W, DOC_CELL_H )];
	[itsDocumentMatrix setIntercellSpacing:NSMakeSize(0,0)];
#else
{	NSButtonCell *buttonCell = [[NSButtonCell new] autorelease];
	[buttonCell setButtonType:NSPushOnPushOffButton];

	itsDocumentMatrix = [[[NSMatrix alloc] initWithFrame:NSMakeRect( DOC_MATRIX_X, DOC_MATRIX_Y, DOC_MATRIX_W, DOC_MATRIX_H )
								mode:NSRadioModeMatrix
								prototype:buttonCell
								numberOfRows:DOC_CELL_CNT_X
								numberOfColumns:DOC_CELL_CNT_Y]
								autorelease];
}
#endif

	[[itsWindow contentView] addSubview:itsDocumentMatrix];

		//	Create the search button
	itsStartButton = [[[NSButton alloc] initWithFrame:NSMakeRect( START_BTN_X, START_BTN_Y, START_BTN_W, START_BTN_H )] autorelease];
	[itsStartButton setButtonType:NSPushOnPushOffButton];
	[itsStartButton setImage:[NSImage imageNamed:@"find_button"]];
	[itsStartButton setImagePosition:NSImageOnly];
	[itsStartButton setTarget:self];
	[itsStartButton setAction:@selector(glibStartQuery:)];
	[[itsWindow contentView] addSubview:itsStartButton];

		//	Create the query field
	itsQueryField = [[[NSTextField alloc] initWithFrame:NSMakeRect( QUERY_FIELD_X, QUERY_FIELD_Y, QUERY_FIELD_W, QUERY_FIELD_H )] autorelease];
	[itsQueryField setTarget:itsStartButton];
	[itsQueryField setAction:@selector(performClick:)];
	[[itsWindow contentView] addSubview:itsQueryField];

[itsQueryField setEditable:YES];
[itsQueryField setStringValue:@"exponential"];

		//	Create the result matrix (which is in a NSScrollView)
	itsResultScrollView = [[[NSScrollView alloc] initWithFrame:NSMakeRect( RESULT_MATRIX_X, RESULT_MATRIX_Y, RESULT_MATRIX_W, RESULT_MATRIX_H )] autorelease];
	[itsResultScrollView setHasVerticalScroller:YES];
	[itsResultScrollView setBorderType:NSBezelBorder];
	[[itsWindow contentView] addSubview:itsResultScrollView];

	itsResultMatrix = [[[NSMatrix alloc] initWithFrame:[itsResultScrollView documentVisibleRect]] autorelease];
	[itsResultMatrix addColumn];
//	[itsResultMatrix setAutosizesCells:YES];
	[itsResultMatrix setCellSize:NSMakeSize( [itsResultScrollView contentSize].width, RESULT_CELL_H )];
	[itsResultMatrix setIntercellSpacing:NSMakeSize(0,0)];
	[itsResultScrollView setDocumentView:itsResultMatrix];
}

//	----------------------------------------------------------------------------

	//	All this is really bad. Matrix is just too row/column oriented for what I need.
- (void)_glibUpdateShelf
{	int theCellCount = [itsDocumentMatrix numberOfRows]*[itsDocumentMatrix numberOfColumns];
	int i;

	for (i=0;i!=theCellCount;i++)
		[self _glibUpdateShelfAt:i];
}

//	----------------------------------------------------------------------------

- (void)_glibUpdateShelfAt:(int)aPosition
{	int x = aPosition%[itsDocumentMatrix numberOfColumns];
	int y = aPosition/[itsDocumentMatrix numberOfColumns];
	NSButtonCell *theCell = [itsDocumentMatrix cellAtRow:y column:x];
	NSArray *theTargets = [itsDocument glibTargets];
	GLibTarget *theTarget = nil;

	if (aPosition<[theTargets count])
		theTarget = [theTargets objectAtIndex:aPosition];

	[self _glibConfigureCell:theCell forTarget:theTarget];
}

//	----------------------------------------------------------------------------

- (void)_glibConfigureCell:(id)aCell forTarget:(GLibTarget *)aTarget
{	if (!aTarget)
	{	[aCell setTitle:@""];
		[aCell setImage:nil];
//		[aCell setEnabled:NO];
		[aCell setHighlightsBy:NSNoCellMask];
	}
	else
	{	[aCell setTitle:[aTarget glibTitle]];
		[aCell setImage:[aTarget glibIcon]];
//		[aCell setEnabled:YES];
		[aCell setHighlightsBy:NSChangeBackgroundCellMask];
	}
}

//	----------------------------------------------------------------------------

- (void)_glibStartObservingDocument:(GLibDocument *)aDocument
{	if (aDocument)
		[aDocument glibAddObserver:self selector:@selector(_glibDocumentChanged:)];
}

//	----------------------------------------------------------------------------

- (void)_glibStopObservingDocument:(GLibDocument *)aDocument
{	if (aDocument)
		[aDocument glibRemoveObserver:self];
}

//	----------------------------------------------------------------------------

- (void)_glibDocumentChanged:(NSNotification *)aNotification
{		//	Check notification is sane
	if ([aNotification object]!=itsDocument)
	{		//	This should not occur, so this log will help trap race condition bugs.
		NSLog( @"**** Notification did not come from controlled object -- Ignored" );
		return;
	}

		//	We should check what part of the object have changed. For various reason, it is not as trivial as we may like
	[self glibUpdateUserInterface];
}

//	----------------------------------------------------------------------------

- (void)_glibStartObservingResultGroupArray:(NSArray *)anArray
{	NSEnumerator *e = [anArray objectEnumerator];
	GLibQueryResultGroup *theGroup;

	while (theGroup=[e nextObject])
		[theGroup glibAddObserver:self selector:@selector(_glibResultGroupChanged:)];
}

//	----------------------------------------------------------------------------

- (void)_glibStopObservingResultGroupArray:(NSArray *)anArray
{	NSEnumerator *e = [anArray objectEnumerator];
	GLibQueryResultGroup *theGroup;

	while (theGroup=[e nextObject])
		[theGroup glibRemoveObserver:self];
}

//	----------------------------------------------------------------------------

//	This may look a bit contrived but the rationale is that I want to make only full update of query result groups (incremental updates are possible, but would be quite difficult for a very general case). The goals are
//	1/ To gain speed 'feel', update results as soon as a query is finished (so short queries look instantaneous)
//	2/ To keep avalaible cycles, delay drawing of partial results
//	3/ To avoid uneeded drawing, dont start an update if one is already pending
//	The point 3 is implemented by having update canceling furture calls
- (void)_glibResultGroupChanged:(NSNotification *)aNotification
{	NSString *theReason = [[aNotification userInfo] objectForKey:GLibQueryResultGroupNotificationReason];

		//	We only have 'more results'. There is nothing really urgent about that
	if ([theReason isEqual:GLibQueryResultGroupNotificationReasonMoreResults])
	{		//	Partial results are drawn 0.5 seconds after they are received
		[self performSelector:@selector(_glibUpdateResultInterface:) withObject:nil afterDelay:0.5];
		return;
	}

		//	Major work have been done by the result group. Show results as soon as possible.
	[self performSelector:@selector(_glibUpdateResultInterface:) withObject:nil afterDelay:0];

		//	See if there are pending executors
	[self _glibCheckForTermination];
}

//	----------------------------------------------------------------------------

- (BOOL)_glibInterfaceToQuery 
{	[(GLibSimpleQuery *)itsQuery glibSetString:[itsQueryField stringValue]];
	return YES;
}

//	----------------------------------------------------------------------------

- (void)_glibInterfaceToActiveTargets
{	int i;

	[itsActiveTargets release];
	itsActiveTargets = [[NSMutableArray alloc] init];

	for (i=0;i!=[[itsDocument glibTargets] count];i++)
	{	int x = i%[itsDocumentMatrix numberOfColumns];
		int y = i/[itsDocumentMatrix numberOfColumns];
		NSButtonCell *theCell = [itsDocumentMatrix cellAtRow:y column:x];
		if ([theCell isHighlighted])
			[itsActiveTargets addObject:[[itsDocument glibTargets] objectAtIndex:i]];
	}
}

//	----------------------------------------------------------------------------

- (void)_glibUpdateResultInterface:sender
{	NSLog( @"_glibUpdateResultInterface:" );

		//	We rebuild all the result matrix [overkill]
	[self _glibRebuildResultMatrix];

		//	We cancel all pending updates #### No, this is bad
	[NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(_glibUpdateResultInterface:) object:nil];
}

//	----------------------------------------------------------------------------

- (void)_glibRebuildResultMatrix 
{	NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];
	NSEnumerator *e = [itsResultGroups objectEnumerator];
	GLibQueryResultGroup *theGroup;


		//	We clear the matrix
	[itsResultMatrix renewRows:0 columns:1];

//	We append each group
	while (theGroup=[e nextObject])
	{	NSAutoreleasePool *theInnerPool = [[NSAutoreleasePool alloc] init];
		[self _glibAppendResultGroup:theGroup];
		[theInnerPool release];
	}
		//	We resize the matrix
//	[itsResultMatrix sizeToCells];
//	[itsResultMatrix sizeToFit];
	[itsResultMatrix setCellSize:NSMakeSize( [itsResultScrollView contentSize].width, RESULT_CELL_H )];
	[itsResultMatrix sizeToCells];

//	GSTEP
//	[itsResultMatrix setNeedsDisplay:YES];
	[itsResultMatrix display];

	[thePool release];
}

//	----------------------------------------------------------------------------

	//	This could probably done by the result group itself (or a result group controller)
- (void)_glibAppendResultGroup:(GLibQueryResultGroup *)aGroup
{	int theCurrentRowCount = [itsResultMatrix numberOfRows];
	NSButtonCell *theTitleCell;
	NSArray *theArray = [aGroup glibResults];
	int i;

		//	Add the title of the result group [We should get the cell from the result group, so we could have much more information in it, like the time spend in query, and so on...]
	[itsResultMatrix renewRows:theCurrentRowCount+[theArray count]+1 columns:1];

	theTitleCell = [[[NSButtonCell alloc] init] autorelease];
	[theTitleCell setEnabled:NO];
	[theTitleCell setTitle:[[aGroup glibTarget] glibTitle]];
	[theTitleCell setAlignment:NSCenterTextAlignment];
	[theTitleCell setBordered:NO];
	[theTitleCell setWraps:NO];
	[itsResultMatrix putCell:theTitleCell atRow:theCurrentRowCount column:0];
//	[theTitleCell release];

		//	Add all the results
	for (i=0;i!=[theArray count];i++)
	{	GLibQueryResult *theResult = [theArray objectAtIndex:i];
			//	Get shared cell that could be used to draw this result	
		NSCell *theCell = [[theResult glibCell] copy];

			//	Configure the cell so it represent the result
		[theResult glibConfigureCell:theCell];

//printf( "S<%p>\n", theSharedCell );
//printf( "C<%p>\n", theCell );
//printf( "M<%p>\n", [itsResultMatrix cellAtRow:theCurrentRowCount+1 column:0] );

	//	Add it to the matrix
		[itsResultMatrix putCell:theCell atRow:++theCurrentRowCount column:0];
//[[itsResultMatrix cellAtRow:++theCurrentRowCount column:0] setStringValue:[theResult glibTitle]];

		[theCell release];
	}
}

//	----------------------------------------------------------------------------

- (void)_glibStartExecutorsForQuery:(GLibQuery *)aQuery resultGroups:(NSArray *)aResultGroupArray
{	NSEnumerator *e = [aResultGroupArray objectEnumerator];
	GLibQueryResultGroup *theResult;

	while (theResult=[e nextObject])
	{	GLibQueryExecutor *theExecutor = [[theResult glibTarget] glibQueryExecutorForQuery:aQuery];
		[itsExecutors addObject:theExecutor];
		[theExecutor glibSetQueryResultGroup:theResult];
		[theExecutor glibStart];
	}
}

//	----------------------------------------------------------------------------

- (void)_glibCheckForTermination
{	NSEnumerator *e = [itsResultGroups objectEnumerator];
	GLibQueryResultGroup *theResult;

		//	See if all result group have finished
	while (theResult=[e nextObject])
	{	if (![theResult glibFinished])
			return ;
	}

		//	Update the interface
	[itsStartButton setImage:[NSImage imageNamed:@"find_button"]];
	[itsStartButton setAction:@selector(glibStartQuery:)];
	[itsStartButton setEnabled:YES];
	[itsExecutors release];
	itsExecutors = nil;
	[self _glibRebuildResultMatrix];
}

//	----------------------------------------------------------------------------

- (void)_glibAbortExecutors
{	NSEnumerator *e = [itsExecutors objectEnumerator];
	GLibQueryExecutor *theExecutor;

	while (theExecutor=[e nextObject])
		[theExecutor glibStop];
}

@end
