/* GLibGlimpseTarget.m created by stark on Sun 13-Dec-1998 */

/* 
   GLibGlimpseTarget.m

   GLibGlimpseTarget for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibGlimpseTarget.h"

#import "GLibQuery.h"
#import "GLibTaskQueryExecutor.h"
#import "GLibQueryResult.h"

	//	This is needed for GNUstep
#import <Foundation/NSScanner.h>

//	----------------------------------------------------------------------------

//	This is a private subclass of GLibFileQueryResult tailored for glimpse output.
//	[This is just a sample. In reality, the default file result should support context, and much more]
//	See [GLibGlimpseTarget glibOutputDidReceiveLine:] for use
@interface GLibGlimpseFileQueryResult : GLibFileQueryResult
{		//	The 'context' glimpse returned
	NSString *itsContext;
}
	//	Init the result based on the string glimpse returned
- initWithString:(NSString *)aString;
@end

//	----------------------------------------------------------------------------

//	This is a private subclass of the query executor that handle the generation of different classes of result
//	As said before, there should nop need to do that for such a simple application (we can use something like prototypes of queryResultClass).
//	Subclassing should only be necessary if subclass need a different functionality (ie: if an output line does not corresponds to one result, or if results kind can change from lines to lines)
@interface GLibGlimpseTaskQueryExecutor : GLibTaskQueryExecutor
@end

//	----------------------------------------------------------------------------

#define KEY_INDEX_DIRECTORY	@"IndexDirectory"

@implementation GLibGlimpseTarget

- initWithDictionary:(NSDictionary *)aConfigDict
{	[super initWithDictionary:aConfigDict];
	itsIndexDirectory = [[aConfigDict objectForKey:KEY_INDEX_DIRECTORY] copy];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[itsIndexDirectory release];
	[super dealloc];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibGlimpseTarget(GLibQueryExecution)

- (GLibQueryExecutor *)glibQueryExecutorForQuery:(GLibQuery *)aQuery
{
	GLibGlimpseTaskQueryExecutor *theExecutor = [[[GLibGlimpseTaskQueryExecutor alloc] init] autorelease];

	[theExecutor glibSetLaunchPath:@"/usr/bin/glimpse"];
	[theExecutor glibAddArgument:@"-i"];
	[theExecutor glibAddArgument:@"-w"];
	[theExecutor glibAddArgument:@"-H"];
	[theExecutor glibAddArgument:[NSString stringWithFormat:@"%@", itsIndexDirectory]];
	[theExecutor glibAddArgument:[(GLibSimpleQuery*)aQuery glibString]];

	return theExecutor;
}

@end

//	----------------------------------------------------------------------------

@implementation GLibGlimpseTaskQueryExecutor
@end

//	----------------------------------------------------------------------------

@implementation GLibGlimpseTaskQueryExecutor(GLibTaskQueryExecutor_Overrides)

- (void)glibOutputDidReceiveLine:(NSString *)aString
{	GLibGlimpseFileQueryResult *theResult = [[[GLibGlimpseFileQueryResult alloc] initWithString:aString] autorelease];
	if (!theResult)		//	We didn't understood the output of glimpse
		theResult = [[[GLibQueryResultError alloc] initWithErrorDescription:aString] autorelease];
	[self glibAppendResult:theResult];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibGlimpseFileQueryResult

- initWithString:(NSString *)aString
{	NSString *thePath=nil;
		//	Scanners IMHO produce convoluted code
	NSScanner *theScanner = [[NSScanner alloc] initWithString:aString];
	if (![theScanner scanUpToString:@":" intoString:&thePath])
		return nil;

	[super initWithPath:thePath];
	[theScanner scanString:@":" intoString:NULL];

	itsContext = nil;
	[theScanner scanUpToString:@"" intoString:&itsContext];
	[itsContext retain];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[itsContext release];
	[super dealloc];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibGlimpseFileQueryResult(GLibQueryResult_Overrides)

- (void)glibConfigureCell:(id)aCell
{		//	Not very elegant
	[super glibConfigureCell:aCell];
	[aCell setTitle:[NSString stringWithFormat:@"%@ : %@", [aCell title], itsContext]];
}

@end
