/* GLibLocateTarget.m created by stark on Mon 09-Nov-1998 */

/* 
   GLibLocateTarget.m

   GLibLocateTarget for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibLocateTarget.h"

#import "GLibQuery.h"
#import "GLibTaskQueryExecutor.h"
#import "GLibNotification.h"

//	----------------------------------------------------------------------------

#define KEY_DATABASE_PATH	@"DatabasePath"

@implementation GLibLocateTarget

- initWithDictionary:(NSDictionary *)aConfigDict
{	[super initWithDictionary:aConfigDict];
	itsDatabasePath = [[aConfigDict objectForKey:KEY_DATABASE_PATH] copy];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[itsDatabasePath release];
	[super dealloc];
}

//	----------------------------------------------------------------------------

+ (NSString *)glibBinaryPath
{	return @"/usr/bin/locate";
}

//	----------------------------------------------------------------------------

- (NSString *)glibDatabasePath
{	return itsDatabasePath;
}

//	----------------------------------------------------------------------------

- (void)glibSetDatabasePath:(NSString *)aDatabasePath
{	[itsDatabasePath autorelease];
	itsDatabasePath = [aDatabasePath copy];
	[self glibObjectChanged];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibLocateTarget(GlibAppearance)

- (NSString *)glibDefaultTitle
{	return itsDatabasePath?[itsDatabasePath lastPathComponent]:@"Standard Locate";
}

@end

//	----------------------------------------------------------------------------

@implementation GLibLocateTarget(GLibQueryExecution)

- (GLibQueryExecutor *)glibQueryExecutorForQuery:(GLibQuery *)aQuery
{	GLibTaskQueryExecutor *theExecutor = [[[GLibTaskQueryExecutor alloc] init] autorelease];

	[theExecutor glibSetLaunchPath:[[self class] glibBinaryPath]];
	if ([self glibDatabasePath])
		[theExecutor glibAddArgument:[NSString stringWithFormat:@"--database=%@", [self glibDatabasePath]]];
		//	We know that the query is 'simple'
		//	The cast is IMHO more powerfull than a isKindOfClass:
		//	#### But this is _not_ the way to do this. Typicall multimethod trick would be better (ie: calling glibQueryExecutorForLocateTarget: on the query). Will only make sense the day we have more complex queries.
	[theExecutor glibAddArgument:[(GLibSimpleQuery*)aQuery glibString]];

	return theExecutor;
}

@end
