/* GLibQueryExecutor.h created by stark on Mon 09-Nov-1998 */

/* 
   GLibQueryExecutor.h

   GLibQueryExecutor for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import <AppKit/AppKit.h>

@class GLibQueryResult;
@class GLibQueryResultGroup;

//	Those objects execute queries.
//	This executor is attached to a query result group, where it adds its results. When the query is finished, he should detach itself from the result group.
//	Each module can build its own subclass of the executor.
@interface GLibQueryExecutor : NSObject
{		//	The [non retained] query result group
	GLibQueryResultGroup *itsResultGroup;
}

	//	Return result group
- (GLibQueryResultGroup *)glibQueryResultGroup;

	//	Change query result group
- (void)glibSetQueryResultGroup:(GLibQueryResultGroup *)aQueryResultGroup;

	//	Start the query. Externally called. Should be subclassed to actually start the query. Return as soon as possible.
- (void)glibStart;

	//	Stop the query. Same as glibStart
- (void)glibStop;

	//	Called when the query stop. This should not be called, but subclassed to provide cleanup for glibStop.
- (void)glibInternalAbort;

	//	End the query. For subclasser use (ie: when a subclasser detect its query is finished normally, he must calls this). This will not call glibInternalAbort.
- (void)glibEnded;

	//	Notify that query aborted (ie: signal an error). For subclasser use. This is similar to glibEnded. This will not call glibInternalAbort.
- (void)glibAbortedWithErrorMessage:(NSString *)anErrorMessage;

	//	Add result to current result set. For subclassers use.
- (void)glibAppendResult:(GLibQueryResult *)aResult;

@end
