/* GLibQueryResult.h created by stark on Mon 09-Nov-1998 */

/* 
   GLibQueryResult.h

   GLibQueryResult for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>		//	This is quite a shame to have this into model objects...

//	This represent a single line of result from a query
//	#### Bookmark may be a much sensible name than 'query result'
@interface GLibQueryResult : NSObject
	//	Return the shared cell for classes that don't override glibCell. A subclass can override only this to provide a cell used for all its instances.
+ (NSCell *)glibSharedCell;

	//	Return a cell suitable for the display of this result. By default it is a NSButtonCell that will use the miniIcon and the title of the query result, so subclasses have an easy way to get displayed
- (NSCell *)glibCell;

	//	Return the title of the result. This can be overriden if glibCell was not
- (NSString *)glibTitle;

	//	Return a mini icon (about 13x13) for this result
- (NSImage *)glibMiniIcon;

	//	Called to configure the cell to display itself. This should be overriden if another kind of cell is used. By default, sets the miniIcon and title into a NSButtonCell
- (void)glibConfigureCell:(id)aCell;

@end

//	This represent an error result
@interface GLibQueryResultError : GLibQueryResult
{		//	The error description
	NSString *itsErrorDescription;
}

	//	Create an error with specific description
- initWithErrorDescription:(NSString *)aDescription;

	//	Description of the error
- (NSString *)glibErrorDescription;

@end

//	This represent a file result [this is really simplistic. We probably need an URL concept somewhere]
@interface GLibFileQueryResult : GLibQueryResult
{		//	The full path
	NSString *itsPath;

		//	The (cached) icon for file
	NSImage *itsMiniIcon;
}

	//	Create a file query result with a specific path
- initWithPath:(NSString *)aPath;

	//	Return path
- (NSString *)glibPath;

@end
