/* GLibQueryResultGroup.h created by stark on Mon 09-Nov-1998 */

/* 
   GLibQueryResultGroup.h

   GLibQueryResultGroup for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import <Foundation/Foundation.h>

@class GLibQueryExecutor;
@class GLibQueryResult;
@class GLibTarget;

//	This object encapsulate results from one target. Query executors add results to the group. (Normally there is only one executor for a GLibQueryResultGroup for the current UI, but this is not enforced by the design. [we could have a 'or' query result where results from two queries would be appened])
@interface GLibQueryResultGroup : NSObject
{
		//	The target [non retained]
	GLibTarget *itsTarget;

		//	The results recevied so far
	NSMutableArray *itsResults;

		//	The number of active executors
	int itsExecutorCount;

		//	The batch count. If non-zero, then batching is in progress
	int itsBatchCount;

		//	If YES, a notification should be sent when batching end
	BOOL itsfNotificationPending;
}

	//	Return target of the result group
- (GLibTarget *)glibTarget;

	//	Set the target of this result group
- (void)glibSetTarget:(GLibTarget *)aTarget;

	//	Current results [really a mutable array]
- (NSArray *)glibResults;

	//	Return YES if there are no executor bounds to this result group
- (BOOL)glibFinished;

	//	Called by executors when they attach
- (void)glibAttachExecutor:(GLibQueryExecutor *)anExecutor;

	//	This can be called if several results are going to be added for better performance. It must be paired with glibEndBatch. Batch can be nested (this is probably useless)
	//	Note that batches are not dependant on the executor.
- (void)glibBeginBatch;

	//	End of a batch
- (void)glibEndBatch;

	//	Called by executors when they want to add results to the query
- (void)glibAddResult:(GLibQueryResult *)aResult forQueryExecutor:(GLibQueryExecutor *)anExecutor;

	//	Called by executors when the query is completed
- (void)glibDetachExecutor:(GLibQueryExecutor *)anExecutor;

@end

//	----------------------------------------------------------------------------

	//	When GLibQueryResultGroup send notifications, the user info contains a string for this key
extern NSString *GLibQueryResultGroupNotificationReason;

	//	This string indicates that the update is due to a few more result added to the result group
extern NSString *GLibQueryResultGroupNotificationReasonMoreResults;

	//	This string indicates that the update is due to a executor that detached itself
extern NSString *GLibQueryResultGroupNotificationReasonExecutorDetached;

