/* GLibQueryResultGroup.m created by stark on Mon 09-Nov-1998 */

/* 
   GLibQueryResultGroup.m

   GLibQueryResultGroup for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibQueryResultGroup.h"

#import "GLibNotification.h"

NSString *GLibQueryResultGroupNotificationReason = @"GLibQueryResultGroupNotificationReason";
NSString *GLibQueryResultGroupNotificationReasonMoreResults = @"GLibQueryResultGroupNotificationReasonMoreResults";
NSString *GLibQueryResultGroupNotificationReasonExecutorDetached = @"GLibQueryResultGroupNotificationReasonExecutorDetach";

//	----------------------------------------------------------------------------

@interface GLibQueryResultGroup(GLibPrivate)
	//	Send notification of 'more results'
- (void)_glibNotifyMoreResult;
@end

//	----------------------------------------------------------------------------

@implementation GLibQueryResultGroup

- init
{	[super init];
	itsResults = [[NSMutableArray alloc] init];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[itsResults release];
	[super dealloc];
}

//	----------------------------------------------------------------------------

- (GLibTarget *)glibTarget
{	return itsTarget;
}

//	----------------------------------------------------------------------------

- (void)glibSetTarget:(GLibTarget *)aTarget
{	itsTarget = aTarget;
}

//	----------------------------------------------------------------------------

- (NSArray *)glibResults
{	return itsResults;
}

//	----------------------------------------------------------------------------

- (BOOL)glibFinished
{	return itsExecutorCount==0;
}

//	----------------------------------------------------------------------------

- (void)glibAttachExecutor:(GLibQueryExecutor *)anExecutor
{	itsExecutorCount++;
}

//	----------------------------------------------------------------------------

- (void)glibBeginBatch
{	itsBatchCount++;
}

//	----------------------------------------------------------------------------

- (void)glibEndBatch
{	if (!--itsBatchCount && itsfNotificationPending)
	{	[self _glibNotifyMoreResult];
		itsfNotificationPending = NO;
	}
}

//	----------------------------------------------------------------------------

- (void)glibAddResult:(GLibQueryResult *)aResult forQueryExecutor:(GLibQueryExecutor *)anExecutor
{	[itsResults addObject:aResult];
	if (!itsBatchCount)
		[self _glibNotifyMoreResult];
	else
		itsfNotificationPending = YES;
}

//	----------------------------------------------------------------------------

- (void)glibDetachExecutor:(GLibQueryExecutor *)anExecutor
{	itsExecutorCount--;	
	[self glibObjectChanged:[NSDictionary dictionaryWithObject:GLibQueryResultGroupNotificationReasonExecutorDetached forKey:GLibQueryResultGroupNotificationReason]];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibQueryResultGroup(GLibPrivate)

- (void)_glibNotifyMoreResult;
{	[self glibObjectChanged:[NSDictionary dictionaryWithObject:GLibQueryResultGroupNotificationReasonMoreResults forKey:GLibQueryResultGroupNotificationReason]];
}

@end
