/* GLibTarget.h created by stark on Wed 04-Nov-1998 */

/* 
   GLibTarget.h

   GLibTarget for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import <Foundation/Foundation.h>

@class GLibQueryExecutor;
@class GLibQuery;

//	There is one instance of a concrete subclass of this class for each searchable target.
//	This is an abstract class
@interface GLibTarget : NSObject
{		//	The title that user gave to this target
	NSString *itsUserTitle;

		//	The image used to display this target.
	id itsDefaultIcon;
}

	//	Initialize with a dictionary. 
- initWithDictionary:(NSDictionary *)aConfigDict;

	//	Return the dictionary object that can represent this target when saving
- (NSDictionary *)glibDictionary;

	//	Should be overriden to add the private keys to the mutable dictionary. Should call super.
- (void)glibAddPrivateKeysToDictionary:(NSMutableDictionary *)aDictionary;

	//	Return the user title
- (NSString *)glibUserTitle;

	//	Change the title
- (void)glibSetUserTitle:(NSString *)aTitle;

/*	Not IMP YET
	//	[#### This is stuff is temporary for generic inspectors, which are needed in the first version. Those objects should init themselves from <drag> pasteboard, or present a private user-interface]
	//	Return a user-editable string that configure this target
- (NSString *)glibConfigurationString;

	//	Change the object configuration based on the passed string
- (void)glibSetConfigurationString:(NSString *)aString;

	//	This can be called by an implementor to signal a change that should be saved to disk
- (void)glibDefinitionChanged;

	//	This can be called by an implementor to signal a change that do not need to be saved, but that may change the appearance of the target (For instance, the target went from non-indexed to indexed, and this is represented differently in the interface)
- (void)glibAppearanceChanged;

	//	This returns an array with the query classes supported by the target. By default this only contains GLibSimpleQuery
- (NSArray *)glibSupportedQueryClasses;
*/

@end

//	----------------------------------------------------------------------------

#import <AppKit/AppKit.h>		//	#### This is hideous

@interface GLibTarget(GlibAppearance)

	//	Return the title of the target. Uses the user title. If the user title is nil, then uses glibDefaultTitle that should be implemented by subclasses
- (NSString *)glibTitle;

	//	Return the icon for the target. Return an image of the same name as the class by default. Can be overriden to provide icons different for different instances of a target class. 
- (NSImage *)glibIcon;

	//	Should be implemented by subclasses to give a default title. Must not return nil. Default implementation does not return nil, so it is safe for subclassers to return super if they don't have any meaningfull ideas.
- (NSString *)glibDefaultTitle;

@end

//	----------------------------------------------------------------------------

//	This category defines the query interface
@interface GLibTarget(GLibQueryExecution)

	//	This returns an (autoreleased) query executor for a specific query. Implementors are free to replace the query object by an equivalent one. This query executor encapsulate all the query details. See query executor class.
- (GLibQueryExecutor *)glibQueryExecutorForQuery:(GLibQuery *)aQuery;

@end
