/* GLibTarget.m created by stark on Wed 04-Nov-1998 */

/* 
   GLibTarget.m

   GLibTarget for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibTarget.h"

#import "GLibNotification.h"

#define KEY_USER_TITLE	@"UserTitle"

//	----------------------------------------------------------------------------

@implementation GLibTarget

- initWithDictionary:(NSDictionary *)aConfigDict
{	[super init];
	[self glibSetUserTitle:[aConfigDict objectForKey:KEY_USER_TITLE]];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	[itsUserTitle release];
	[itsDefaultIcon release];
	[super dealloc];
}

//	----------------------------------------------------------------------------

- (NSDictionary *)glibDictionary
{		//	Create dict
	NSMutableDictionary *theDict = [[NSMutableDictionary alloc] init];

		//	Add keys
	[self glibAddPrivateKeysToDictionary:theDict];

		//	We could copy it, but it is not strictly necessary
	return theDict;
}

//	----------------------------------------------------------------------------

- (void)glibAddPrivateKeysToDictionary:(NSMutableDictionary *)aDictionary
{		//	Subclasses should call super before adding their keys
	if (itsUserTitle)
		[aDictionary setObject:itsUserTitle forKey:KEY_USER_TITLE];
}

//	----------------------------------------------------------------------------

- (NSString *)glibUserTitle;
{	return itsUserTitle;
}

//	----------------------------------------------------------------------------

- (void)glibSetUserTitle:(NSString *)aTitle
{	[itsUserTitle autorelease];
	itsUserTitle = [aTitle copy];
	[self glibObjectChanged];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibTarget(GlibAppearance)

- (NSString *)glibTitle
{		//	First look ifd there is a user title
	NSString *theTitle = [self glibUserTitle];

	if (!theTitle)
	{		//	Second ask the subclasses for a default title
		theTitle = [self glibDefaultTitle];

		assert( theTitle );
	}

	return theTitle;
}

//	----------------------------------------------------------------------------

- (NSString *)glibDefaultTitle
{	return NSStringFromClass( [self class] );
}

//	----------------------------------------------------------------------------

- (NSImage *)glibIcon
{	NSImage *theImage = nil;
	NSString *theImagePath;
	NSString *theImageName = NSStringFromClass( [self class] );

	if (itsDefaultIcon)
		return itsDefaultIcon;

		//	XRaw is very slow to cache images
	if (!theImage)
		theImage = [NSImage imageNamed:theImageName];

		//	We have to find a suitable image
//	#### Next line does not work on GNUstep (maybe it don't find tiffs)
//	theImagePath = [[NSBundle bundleForClass:[self class]] pathForImageResource:NSStringFromClass([self class])];

	if (!theImage)
	{	theImagePath = [[NSBundle bundleForClass:[self class]] pathForResource:NSStringFromClass([self class]) ofType:@"tiff"];

			//	Okay, we have a path to it
		if (theImagePath)
		{	theImage = [[[NSImage alloc] initByReferencingFile:theImagePath] autorelease];
				//	Mmm. We could check the the image is valid by using lockFocus. If invalid we should set the image to nil...
			[theImage setName:theImageName];	//	#### Probably not necessary. Check this
		}
	}

		//	If we still don't have an image
	if (!theImage)
	{	NSLog( @"-[GLibTarget glibIcon] : NO ICON FOR CLASS [%@], USING DEFAULT", NSStringFromClass( [self class] ) );
		theImage = [NSImage imageNamed:@"unknown_target"];
	}

		//	We hope it was enough
	assert( theImage );

	itsDefaultIcon = [theImage retain];

	return itsDefaultIcon;
}

@end
