/* GLibTaskQueryExecutor.h created by stark on Sun 13-Dec-1998 */

/* 
   GLibTaskQueryExecutor.h

   GLibTaskQueryExecutor for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#import "GLibQueryExecutor.h"

#import <Foundation/NSTask.h>
#import <Foundation/NSFileHandle.h>
//#import <Foundation/NSPipe.h>

//	This is a query executor that execute a NSTask to get the query results
//	The executor listen for taks standard output and error output and terminate the query when the NSTask is teminated and both stream are closed
//	By default, both the standard output and error are broken in lines wich can be processed by subclasses. A subclass can change this default behaviour
@interface GLibTaskQueryExecutor : GLibQueryExecutor
{
		//	The spwaned task
	NSTask *itsTask;

		//	The pipes (output and error)
	NSPipe *itsPipes[2];

		//	The 'current' splitted line of standard output|error (which is really part of next ouput). This is often the empty (@"") string
	NSString *itsSplitLine[2];

		//	The number of event we wait for. (Task termination, standard output and error output end of stream)
	int itsState;

		//	The launch path
	NSString *itsLaunchPath;

		//	The arguments
	NSMutableArray *itsArguments;
}

	//	Create the task (but don't launch it)
- (NSTask *)glibCreateTask;

	//	Return the launch path of sub task
- (NSString *)glibLaunchPath;

	//	Change the launch path of the sub task
- (void)glibSetLaunchPath:(NSString *)aPath;

	//	Return the arguments for sub task
- (NSArray *)glibArguments;

	//	Add an argument for the sub task
- (void)glibAddArgument:(NSString *)anArgument;

	//	Called when output/error stream receive data. By default convert the data into a string and call glibDidReceiveLine:onPipe:
- (void)glibDidReceiveData:(NSData *)aData onPipe:(int)aPipeNumber;

	//	Called by glibDidReceiveData:onPipe: after having converted output to a string. By default, split this string into lines, begin a batch for the result group, and call glib(Output|Error)DidReceiveLine: with each string
- (void)glibDidReceiveString:(NSString *)aString onPipe:(int)aPipeNumber;

	//	Called by glibDidReceiveString:onPipe: after splitting string into lines for standard output. Consider lines as files by default, so queries that only output file names on their standard output are very easy to implement
- (void)glibOutputDidReceiveLine:(NSString *)aString;

	//	Called by glibDidReceiveString:onPipe: after splitting string into lines for standard error. Consider lines as self-contained error/warning messages by default
- (void)glibErrorDidReceiveLine:(NSString *)aString;

	//	Called when the output stream is closed (it is rather risky to override)
- (void)glibOutputDidClose;

	//	Called when the error stream is closed (it is rather risky to override)
- (void)glibErrorDidClose;

@end
