/* GLibTaskQueryExecutor.m created by stark on Sun 13-Dec-1998 */

/* 
   GLibTaskQueryExecutor.m

   GLibTaskQueryExecutor for GNUstep GNUlibrarian

   Copyright (C) 1998 Free Software Foundation, Inc.

   Author:  Frederic Stark <stark@easynet.fr>
   Date: December 1998
   
   This file is part of the GNUstep GNUlibrarian.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

//	#### Side note: NSFileHandleDataAvailableNotification is not implemented in GNUstep

#import "GLibTaskQueryExecutor.h"

#import "GLibQueryResult.h"
#import "GLibQueryResultGroup.h"

//	----------------------------------------------------------------------------

@interface GLibTaskQueryExecutor(GLibPrivate)
	//	Increment state
- (void)_glibIncState;
	//	Decrement state
- (void)_glibDecState;
	//	Notification from NSTask
- (void)_glibTaskFinished:(NSNotification *)aNotification;
	//	Notification from standard output
- (void)_glibTaskOuput:(NSNotification *)aNotification;
	//	Notification from standard error
- (void)_glibTaskError:(NSNotification *)aNotification;
	//	Clean up a finished query
- (void)_glibCleanUp;

@end

//	----------------------------------------------------------------------------

@implementation GLibTaskQueryExecutor

- init
{	[super init];
	itsArguments = [[NSMutableArray alloc] init];
	return self;
}

//	----------------------------------------------------------------------------

- (void)dealloc
{	//	assert( itsState== 0 );	#### The fact we can close with non 0 state smell bad
	[itsLaunchPath release];
	[itsArguments release];
	[super dealloc];
}

//	----------------------------------------------------------------------------

- (NSTask *)glibCreateTask
{		//	Create task
	NSTask *theTask = [[NSTask alloc] init];

		//	Set executable
	[theTask setLaunchPath:[self glibLaunchPath]];

		//	Set arguments
	[theTask setArguments:[self glibArguments]];

	return theTask;
}

//	----------------------------------------------------------------------------

- (NSString *)glibLaunchPath
{	return itsLaunchPath;
}

//	----------------------------------------------------------------------------

- (void)glibSetLaunchPath:(NSString *)aPath
{	[itsLaunchPath autorelease];
	itsLaunchPath = [aPath copy];
}

//	----------------------------------------------------------------------------

- (NSArray *)glibArguments
{	return itsArguments;
}

//	----------------------------------------------------------------------------

- (void)glibAddArgument:(NSString *)anArgument
{	[itsArguments addObject:anArgument];
}

//	----------------------------------------------------------------------------

- (void)glibDidReceiveData:(NSData *)aData onPipe:(int)aPipeNumber
{	NSString *theString = [[NSString alloc] initWithData:aData encoding:NSISOLatin1StringEncoding];

	NS_DURING
		[self glibDidReceiveString:theString onPipe:aPipeNumber];
	NS_HANDLER
		[theString release];		//	We did not autorelease the string, so we need that to be execption safe
	NS_ENDHANDLER
}

//	----------------------------------------------------------------------------

- (void)glibDidReceiveString:(NSString *)aString onPipe:(int)aPipeNumber
{		//	Splitting the array into lines. This is probably costly.
	NSArray *theArray = [aString componentsSeparatedByString:@"\n"];

		//	We deal with the splitted line
	[itsSplitLine[aPipeNumber] autorelease];
	itsSplitLine[aPipeNumber] = [[itsSplitLine[aPipeNumber] stringByAppendingString:[theArray objectAtIndex:0]] retain];

		//	If array count is 0, we still did fininshed our split line
	if ([theArray count]>1)
	{	int i;
			//	Start a batch (ie: the result group will notify at end of the batch)
		[itsResultGroup glibBeginBatch];

			//	Split line
			if (aPipeNumber==0)
				[self glibOutputDidReceiveLine:itsSplitLine[aPipeNumber]];
			else
				[self glibErrorDidReceiveLine:itsSplitLine[aPipeNumber]];

			//	Each new line (but the last)
		for (i=1;i!=[theArray count]-1;i++)
			if (aPipeNumber==0)
				[self glibOutputDidReceiveLine:[theArray objectAtIndex:i]];
			else
				[self glibErrorDidReceiveLine:[theArray objectAtIndex:i]];

			//	End a batch (result group will notify if any new result was appended by glib(Output|Error)DidReceiveLine:)
		[itsResultGroup glibEndBatch];

			//	We have a new split line
		[itsSplitLine[aPipeNumber] release];
		itsSplitLine[aPipeNumber] = [[theArray lastObject] retain];
	}

		//	Ask for next notification
	[[itsPipes[aPipeNumber] fileHandleForReading] readInBackgroundAndNotify];
}

//	----------------------------------------------------------------------------

- (void)glibOutputDidReceiveLine:(NSString *)aString
{	GLibFileQueryResult *theResult = [[[GLibFileQueryResult alloc] initWithPath:aString] autorelease];
	[self glibAppendResult:theResult];
}

//	----------------------------------------------------------------------------

- (void)glibErrorDidReceiveLine:(NSString *)aString
{	GLibQueryResultError *theResult = [[[GLibQueryResultError alloc] initWithErrorDescription:aString] autorelease];
	[self glibAppendResult:theResult];
}

//	----------------------------------------------------------------------------

- (void)glibOutputDidClose
{	[self _glibDecState];
}

//	----------------------------------------------------------------------------

- (void)glibErrorDidClose
{	[self _glibDecState];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibTaskQueryExecutor(GLibQueryExecutor_Overrides)

- (void)glibStart
{	assert( !itsTask );
	itsTask = [self glibCreateTask];

	[super glibStart];

	itsSplitLine[0] = [@"" retain];
	itsSplitLine[1] = [@"" retain];

	itsTask = [self glibCreateTask];

		//	Listen to notifications
    [[NSNotificationCenter defaultCenter] addObserver:self 
		selector:@selector(_glibTaskFinished:) 
		name:NSTaskDidTerminateNotification 
		object:itsTask];

		//	Set standard output
		//	#### This produce a warning on GNUstep because it want a NSFileHandle. OPENSTEP documentation states that a NSPipe is valid too
	itsPipes[0] = [[NSPipe alloc] init];
	[itsTask setStandardOutput:itsPipes[0]];
//GSTEP	[itsTask setStandardOutput:[itsPipes[0] fileHandleForWriting]];

		//	Set standard error
	itsPipes[1] = [[NSPipe alloc] init];
	[itsTask setStandardError:itsPipes[1]];
//GSTEP	[itsTask setStandardError:[itsPipes[1] fileHandleForWriting]];

		//	Wait for data
	[self _glibIncState];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(_glibTaskOuput:)
		name:NSFileHandleReadCompletionNotification
		object:[itsPipes[0] fileHandleForReading]];
	[[itsPipes[0] fileHandleForReading] readInBackgroundAndNotify];

		//	And errors
	[self _glibIncState];
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(_glibTaskError:)
		name:NSFileHandleReadCompletionNotification
		object:[itsPipes[1] fileHandleForReading]];
	[[itsPipes[1] fileHandleForReading] readInBackgroundAndNotify];

		//	Launch
	[self _glibIncState];

	NSLog( @"<%@ %@>", [itsTask launchPath], [[itsTask arguments] componentsJoinedByString:@" "] );
	NS_DURING
		[itsTask launch];
	NS_HANDLER
	{	GLibQueryResultError *theResult;


		NSLog( @"Exception : <%@>", localException );
		NSLog( @"Exception : <%@>", [localException name] );
		NSLog( @"Exception : <%@>", [localException reason] );
		theResult = [[[GLibQueryResultError alloc] initWithErrorDescription:[NSString stringWithFormat:@"Cannot launch subprocess '%@'", [itsTask launchPath]]] autorelease];
		[self _glibDecState];
		[self _glibDecState];
		[self _glibDecState];
		[self glibAppendResult:theResult];
	}
	NS_ENDHANDLER
}

//	----------------------------------------------------------------------------

- (void)glibInternalAbort
{	[super glibInternalAbort];

		//	We remove observers. This is not really clean because data may still come to the applicaiton (if the stop is called into an abort context)
		//	Maybe we should delay the destruction of executors in this case (ie: turn them into a zombie state) until their state goes back to zero. Mmm
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSTaskDidTerminateNotification object:itsTask];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object:[itsPipes[0] fileHandleForReading]];
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object:[itsPipes[1] fileHandleForReading]];

		//	Terminate the task. This is another risky point: the task may go berserk. Same as above: we could stay alive, waiting for termination and take a more agressive action if the task refuse to terminate after a given delay (ie: kill -KILL), or sending a message to the user/loggin to the console.
	[itsTask terminate];

	[self _glibCleanUp];
}

@end

//	----------------------------------------------------------------------------

@implementation GLibTaskQueryExecutor(GLibPrivate)

- (void)_glibIncState
{	itsState++;
}

//	----------------------------------------------------------------------------

- (void)_glibDecState
{	itsState--;
	NSLog( @"STATE %d", itsState );
	if (itsState==0)
		[self glibEnded];
}

//	----------------------------------------------------------------------------

- (void)_glibTaskFinished:(NSNotification *)aNotification
{	NSLog( @"Task query [%@] finished", self );
	[self _glibDecState];
}

//	----------------------------------------------------------------------------

- (void)_glibTaskOuput:(NSNotification *)aNotification
{	NSData *theData = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];

	if ([theData length]==0)
	{	[self glibOutputDidClose];
		return ;
	}

	[self glibDidReceiveData:theData onPipe:0];
}

//	----------------------------------------------------------------------------

- (void)_glibTaskError:(NSNotification *)aNotification
{	NSData *theData = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];

	if ([theData length]==0)
	{	[self glibErrorDidClose];
		return ;
	}

	[self glibDidReceiveData:theData onPipe:1];
}

//	----------------------------------------------------------------------------

- (void)_glibCleanUp
{	[[itsPipes[0] fileHandleForReading] closeFile];	//	This should not be necessary but it looks like i loose filedesc if I don't close. (?)
	[[itsPipes[1] fileHandleForReading] closeFile];

	[itsPipes[0] release];
	[itsPipes[1] release];
	itsPipes[0] = itsPipes[1] = nil;

	[itsSplitLine[0] release];
	[itsSplitLine[1] release];
	itsSplitLine[0] = itsSplitLine[1] = nil;

	itsTask = nil;
}

@end
